@@ -1,8 +1,8 @@
 package Parse::HTTP::UserAgent;
-use strict;
-use warnings;
+$Parse::HTTP::UserAgent::VERSION = '0.40_02'; # TRIAL
 
-our $VERSION = '0.40_01';
+$Parse::HTTP::UserAgent::VERSION = '0.4002';use strict;
+use warnings;
 
 use base qw(
     Parse::HTTP::UserAgent::Base::IS
@@ -165,20 +165,37 @@ sub _do_parse {
             : index( $_, 'rv:')      != NO_IMATCH ? ( version => 1 )
             : ()
         } @{ $t };
+        my $msie_matched = keys %msie11;
 
-        if ( keys %msie11 == 3 ){
+        if ( $msie_matched == 3 ){
             return $self->_parse_msie_11($m, $t, $e);
         }
+        elsif ( ! $self->[IS_MAXTHON] && $msie_matched == 2 && ! $msie11{version} ) {
+            # another weird case. robot?
+            my(@buf, $vstr);
+            for my $junk ( @{ $t } ) {
+                if ( index( $junk, 'MSIE') != NO_IMATCH ) {
+                    $vstr = $junk;
+                    next;
+                }
+                push @buf, $junk;
+            }
+
+            my $rv = $self->_parse_msie($m, \@buf, $e, split( RE_WHITESPACE, $vstr ) );
+            return $rv;
+        }
+        # fall back to the dispatch table below
     }
 
-    my $rv =  $self->[IS_MAXTHON]        ? [maxthon    => $m, $t, $e, @o       ]
-            : $self->_is_opera_pre($m)   ? [opera_pre  => $m, $t, $e           ]
-            : $self->_is_opera_post($e)  ? [opera_post => $m, $t, $e, $c       ]
-            : $self->_is_opera_ff($e)    ? [opera_pre  => "$e->[2]/$e->[3]", $t]
-            : $self->_is_ff($e)          ? [firefox    => $m, $t, $e, @o       ]
-            : $self->_is_safari($e, \@o) ? [safari     => $m, $t, $e, @o       ]
-            : $self->_is_chrome($e, \@o) ? [chrome     => $m, $t, $e, @o       ]
-            : $self->_is_android($t,\@o) ? [android    => $m, $t, $e, @o       ]
+    my $rv =  $self->[IS_MAXTHON]          ? [ maxthon    => $m, $t, $e, @o        ]
+            : $self->_is_opera_pre($m)     ? [ opera_pre  => $m, $t, $e            ]
+            : $self->_is_opera_post($e)    ? [ opera_post => $m, $t, $e, $c        ]
+            : $self->_is_opera_ff($e)      ? [ opera_pre  => "$e->[2]/$e->[3]", $t ]
+            : $self->_is_ff($e)            ? [ firefox    => $m, $t, $e, @o        ]
+            : $self->_is_safari($e, \@o)   ? [ safari     => $m, $t, $e, @o        ]
+            : $self->_is_chrome($e, \@o)   ? [ chrome     => $m, $t, $e, @o        ]
+            : $self->_is_android($t,\@o)   ? [ android    => $m, $t, $e, @o        ]
+            : $self->_is_suspicious_ff($e) ? [ ff_suspect => $m, $t, $e, @o        ]
             : undef;
 
     if ( $rv ) {
@@ -211,7 +228,7 @@ sub _post_parse {
         push @buf, $e;
     }
 
-    $self->[UA_EXTRAS] = [ @buf ];
+    $self->[UA_EXTRAS] = @buf ? [ @buf ] : undef;
 
     if ( $self->[UA_TOOLKIT] ) {
         my $v = $self->[UA_TOOLKIT][TK_ORIGINAL_VERSION];
@@ -293,6 +310,13 @@ sub _numify {
         push @removed, '-' x $rc if INSIDE_VERBOSE_TEST;
     }
 
+    # convert _ to .
+    # version.pm has changed its interpretation of versions with underlines
+    # cf. https://bugs.debian.org/825611
+    if ( my $rc = $v =~ tr/_/./ ) {
+        push @removed, '-' x $rc if INSIDE_VERBOSE_TEST;
+    }
+
     # Finally, be aggressive to prevent dying on bogus stuff.
     # It's interesting how people provide highly stupid version "numbers".
     # Version parameters are probably more stupid than the UA string itself.
@@ -369,9 +393,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
-Parse::HTTP::UserAgent - Parser for the User Agent string
+Parse::HTTP::UserAgent
+
+=head1 VERSION
+
+version 0.40_02
 
 =head1 SYNOPSIS
 
@@ -386,12 +416,6 @@ Parse::HTTP::UserAgent - Parser for the User Agent string
 
 =head1 DESCRIPTION
 
-This document describes version C<0.40_01> of C<Parse::HTTP::UserAgent>
-released on C<20 January 2014>.
-
-B<WARNING>: This version of the module is part of a
-developer (beta) release of the distribution and it is
-not suitable for production use.
 Quoting L<http://www.webaim.org/blog/user-agent-string-history/>:
 
    " ... and then Google built Chrome, and Chrome used Webkit, and it was like
@@ -418,6 +442,10 @@ MSIE, FireFox, Opera, Safari & Chrome first. It then tries to identify Mozilla,
 Netscape, Robots and the rest will be tried with a generic parser. There is
 also a structure dumper, useful for debugging.
 
+=head1 NAME
+
+Parse::HTTP::UserAgent - Parser for the User Agent string
+
 =head1 METHODS
 
 =head2 new STRING [, OPTIONS ]
@@ -565,15 +593,13 @@ L<http://use.perl.org/~Burak/journal/39577> (4 September 2009).
 
 =head1 AUTHOR
 
-Burak Gursoy <burak@cpan.org>.
+Burak Gursoy <burak@cpan.org>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009 - 2014 Burak Gursoy. All rights reserved.
+This software is copyright (c) 2009 by Burak Gursoy.
 
-=head1 LICENSE
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.16.2 or,
-at your option, any later version of Perl 5 you may have available.
 =cut
