@@ -1,11 +1,11 @@
 package Parse::HTTP::UserAgent::Base::Dumper;
-use strict;
+$Parse::HTTP::UserAgent::Base::Dumper::VERSION = '0.40_02'; # TRIAL
+
+$Parse::HTTP::UserAgent::Base::Dumper::VERSION = '0.4002';use strict;
 use warnings;
 use Carp qw( croak );
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-our $VERSION = '0.40_01';
-
 sub dumper {
     my($self, @args) = @_;
     my %opt = @args % 2 ? () : (
@@ -102,20 +102,24 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
-Parse::HTTP::UserAgent::Base::Dumper - Base class to dump parsed structure
+Parse::HTTP::UserAgent::Base::Dumper
+
+=head1 VERSION
+
+version 0.40_02
 
 =head1 DESCRIPTION
 
-This document describes version C<0.40_01> of C<Parse::HTTP::UserAgent::Base::Dumper>
-released on C<20 January 2014>.
-
-B<WARNING>: This version of the module is part of a
-developer (beta) release of the distribution and it is
-not suitable for production use.
 The parsed structure can be dumped to a text table for debugging.
 
+=head1 NAME
+
+Parse::HTTP::UserAgent::Base::Dumper - Base class to dump parsed structure
+
 =head1 METHODS
 
 =head2 dumper
@@ -129,15 +133,13 @@ L<Parse::HTTP::UserAgent>.
 
 =head1 AUTHOR
 
-Burak Gursoy <burak@cpan.org>.
+Burak Gursoy <burak@cpan.org>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009 - 2014 Burak Gursoy. All rights reserved.
+This software is copyright (c) 2009 by Burak Gursoy.
 
-=head1 LICENSE
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.16.2 or,
-at your option, any later version of Perl 5 you may have available.
 =cut
