@@ -1,10 +1,10 @@
 package Parse::HTTP::UserAgent::Base::Parsers;
-use strict;
+$Parse::HTTP::UserAgent::Base::Parsers::VERSION = '0.40_02'; # TRIAL
+
+$Parse::HTTP::UserAgent::Base::Parsers::VERSION = '0.4002';use strict;
 use warnings;
 use Parse::HTTP::UserAgent::Constants qw(:all);
 
-our $VERSION = '0.40_01';
-
 sub _extract_dotnet {
     my($self, @args) = @_;
     my @raw  = map { ref($_) eq 'ARRAY' ? @{$_} : $_ } grep { $_ } @args;
@@ -43,7 +43,7 @@ sub _fix_opera {
     }
     $self->_fix_os_lang;
     $self->_fix_windows_nt('skip_os');
-    $self->[UA_EXTRAS] = [ @buf ];
+    $self->[UA_EXTRAS] = @buf ? [ @buf ] : undef;
     return 1;
 }
 
@@ -111,7 +111,7 @@ sub _parse_maxthon {
 
     if ( $is_30 ) {
         if ( $self->[UA_LANG] ) {
-            push @{ $self->[UA_EXTRAS] }, $self->[UA_LANG];
+            push @{ $self->[UA_EXTRAS] ||= [] }, $self->[UA_LANG];
             $self->[UA_LANG] = undef;
         }
     }
@@ -198,7 +198,7 @@ sub _parse_msie {
         @buf
     ;
 
-    $self->[UA_EXTRAS] = [ @extras ] if @extras;
+    $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
     $self->[UA_PARSER] = 'msie';
 
     return 1;
@@ -228,16 +228,15 @@ sub _parse_msie_11 {
 
     if ( $self->[UA_TOUCH] && $self->[UA_EXTRAS] ) {
         # version 10+
-        $self->[UA_EXTRAS] = [
-            map {
-                $_ eq 'ARM'
-                    ? do {
-                        $self->[UA_DEVICE] = $_;
-                        ()
-                      }
-                    : $_
-            } @{ $self->[UA_EXTRAS] }
-        ];
+        my @extras = map {
+            $_ eq 'ARM'
+                ? do {
+                    $self->[UA_DEVICE] = $_;
+                    ()
+                  }
+                : $_
+        } @{ $self->[UA_EXTRAS] };
+        $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
     }
 
     $self->[UA_PARSER] = 'msie11';
@@ -261,6 +260,18 @@ sub _parse_firefox {
     return 1;
 }
 
+sub _parse_ff_suspect {
+    my($self, $moz, $thing, $extra, @others) = @_;
+    # fool the moz parser
+    unshift @{ $extra }, '';
+
+    $self->_parse_mozilla_family( $moz, $thing, $extra, @others );
+
+    $self->[UA_PARSER] = 'ff_suspect';
+
+    return 1;
+}
+
 sub _fix_fennec {
     my($self, $e) = @_;
     my($name, $version) = split RE_SLASH, pop @{ $e };
@@ -274,7 +285,7 @@ sub _fix_fennec {
         $self->[UA_LANG]   = undef;
     }
     elsif ( index( $self->[UA_LANG], q{ } ) != NO_IMATCH ) {
-        push @{ $self->[UA_EXTRAS] }, $self->[UA_LANG];
+        push @{ $self->[UA_EXTRAS] ||= [] }, $self->[UA_LANG];
         $self->[UA_LANG] = undef;
     }
     else {
@@ -328,7 +339,7 @@ sub _parse_safari {
 
         if ( $check_os && index( $check_os, 'Mac OS X' ) != NO_IMATCH ) {
             if ( $self->[UA_OS] ) {
-                push @{$self->[UA_EXTRAS]}, $self->[UA_OS];
+                push @{ $self->[UA_EXTRAS] ||= [] }, $self->[UA_OS];
             }
             $self->[UA_OS] = pop @{ $thing };
             # Another oddity: tk as "AppleWebKit/en_SG"
@@ -342,15 +353,24 @@ sub _parse_safari {
         }
     }
 
-    $self->[UA_EXTRAS] = [ @{$thing}, @others ];
+    my @extras;
+    push @extras, @{$thing}, @others;
 
     if ( $self->[UA_OS] && length($self->[UA_OS]) == 1 ) {
-        push @{$self->[UA_EXTRAS]}, $self->[UA_OS];
+        push @extras, $self->[UA_OS];
         $self->[UA_OS] = undef;
     }
 
-    push @{$self->[UA_EXTRAS]}, @junk if @junk;
-    push @{$self->[UA_EXTRAS]}, @{$extra} if $extra;
+    if ( $self->[UA_LANG] && $self->[UA_LANG] !~ m{[a-zA-Z]+}xmsg ) {
+        # some junk like "6.0" -- more stupidity
+        push @extras, $self->[UA_LANG];
+        $self->[UA_LANG] = undef;
+    }
+
+    push @extras, @junk     if @junk;
+    push @extras, @{$extra} if $extra;
+
+    $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
 
     return 1;
 }
@@ -429,10 +449,12 @@ sub _parse_android {
         unshift @extras, join ' ', map { shift @extras } 1..3;
     }
 
+    my @extras_final = grep { $_ } @extras;
+
     $self->[UA_NAME]   = 'Android';
     $self->[UA_MOBILE] = 1;
     $self->[UA_TABLET] = $is_phone ? undef : 1;
-    $self->[UA_EXTRAS] = [ grep { $_ } @extras ];
+    $self->[UA_EXTRAS] = @extras_final ? [ @extras_final ] : undef;
 
     return 1;
 }
@@ -474,12 +496,19 @@ sub _parse_opera_pre {
     }
 
     $self->[UA_LANG] = $lang;
-    $self->[UA_OS]   = @{$thing} && $self->_is_strength( $thing->[LAST_ELEMENT] )
-                     ? shift @{$thing}
-                     : pop   @{$thing}
-                     ;
 
-    $self->[UA_EXTRAS] = [ @{ $thing }, ( $extra ? @{$extra} : () ) ];
+    if ( @{$thing} && $self->_is_strength( $thing->[LAST_ELEMENT] ) ) {
+        $self->[UA_STRENGTH] = pop   @{ $thing };
+        $self->[UA_OS]       = shift @{ $thing };
+    }
+    else {
+        $self->[UA_OS]       = pop   @{ $thing };
+    }
+
+    my @extras =  ( @{ $thing }, ( $extra ? @{$extra} : () ) );
+
+    $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
+
     return $self->_fix_opera;
 }
 
@@ -490,11 +519,17 @@ sub _parse_opera_post {
     $self->[UA_NAME]        = shift @{$extra};
     $self->[UA_VERSION_RAW] = shift @{$extra};
    ($self->[UA_LANG]        = shift @{$extra} || q{}) =~ tr/[]//d;
-    $self->[UA_OS]          = @{$thing} && $self->_is_strength($thing->[LAST_ELEMENT])
-                            ? shift @{$thing}
-                            : pop   @{$thing}
-                            ;
-    $self->[UA_EXTRAS]      = [ @{ $thing }, ( $extra ? @{$extra} : () ) ];
+
+    if ( @{$thing} && $self->_is_strength( $thing->[LAST_ELEMENT] ) ) {
+        $self->[UA_STRENGTH] = pop   @{ $thing };
+        $self->[UA_OS]       = shift @{ $thing };
+    }
+    else {
+        $self->[UA_OS]       = pop   @{ $thing };
+    }
+
+    my @extras = ( @{ $thing }, ( $extra ? @{$extra} : () ) );
+    $self->[UA_EXTRAS]      = @extras ? [ @extras ] : undef;
     return $self->_fix_opera;
 }
 
@@ -511,30 +546,68 @@ sub _parse_mozilla_family {
     $self->[UA_VERSION_RAW]  = $version;
     $self->[UA_TOOLKIT]      = $extra->[0]
                              ? [ split RE_SLASH, shift @{ $extra } ]
-                             : []
+                             : undef
                              ;
 
     if ( @{$thing} && index($thing->[LAST_ELEMENT], 'rv:') != NO_IMATCH ) {
         $self->[UA_MOZILLA]  = pop @{ $thing };
-        if ( @{ $thing } <= 3 ) {
+        my $len_thing = @{ $thing };
+        if ( $len_thing == 3 ) {
             $self->[UA_OS] = shift @{ $thing };
             if ( $self->[UA_OS] && $self->[UA_OS] eq 'Macintosh' ) {
                 $self->[UA_OS] = shift @{ $thing };
             }
             $self->[UA_LANG] = pop @{ $thing } if @{ $thing };
         }
+        elsif ( $len_thing <= 2 ) {
+            if (   $thing->[0] eq 'X11'
+                || index( $thing->[-1], 'Intel' ) != NO_IMATCH
+            ) {
+                if ( index( lc $thing->[-1], 'linux arm') != NO_IMATCH ) {
+                    $self->[UA_DEVICE] = pop @{ $thing };
+                    $self->[UA_OS]     = 'Linux'; # Android? huh?
+                }
+                else {
+                    $self->[UA_OS]   = pop @{ $thing };
+                }
+            }
+            elsif (
+                   index( lc $thing->[0], 'android' ) != NO_IMATCH
+                || index( lc $thing->[0], 'maemo'   ) != NO_IMATCH
+            ) {
+                # mobile? tablet?
+                $self->[UA_OS]     = shift @{ $thing };
+                $self->[UA_DEVICE] = shift @{ $thing };
+                if ( lc $self->[UA_DEVICE] eq 'tablet' ) {
+                    $self->[UA_TABLET] = 1;
+                }
+            }
+            else {
+                if ( $len_thing > 1 ) {
+                    if ( $thing->[-1] ne 'WOW64' ) {
+                        $self->[UA_LANG] = pop @{ $thing };
+                    }
+                }
+                else {
+                    $self->[UA_OS]   = pop @{ $thing };
+                }
+            }
+        }
         else {
+
             $self->[UA_LANG]     = pop @{ $thing };
             $self->[UA_OS]       = pop @{ $thing };
         }
     }
 
-    $self->[UA_EXTRAS] = [
-        grep { $_ }
+    my @extras = grep { $_ }
         @{ $thing },
         @others,
         $extra ? @{ $extra } : (),
-    ];
+    ;
+
+    $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
+
     return 1;
 }
 
@@ -578,13 +651,13 @@ sub _parse_gecko {
             push @buf, $e;
         }
 
-        $self->[UA_EXTRAS]        = [ @buf ];
+        $self->[UA_EXTRAS]        = @buf ? [ @buf ] : undef;
         $self->[UA_ORIGINAL_NAME] = $before if $before ne $self->[UA_NAME];
         $self->_fix_windows_nt;
         return 1 ;
     }
 
-    if ( $self->[UA_TOOLKIT] && $self->[UA_TOOLKIT][TK_NAME] eq 'Gecko' ) {
+    if ( ref $self->[UA_TOOLKIT] eq 'ARRAY' && $self->[UA_TOOLKIT][TK_NAME] eq 'Gecko' ) {
         ($self->[UA_NAME], $self->[UA_VERSION_RAW]) = split RE_SLASH, $moz;
         if ( $self->[UA_NAME] && $self->[UA_VERSION_RAW] ) {
             $self->[UA_PARSER] = 'mozilla_family:gecko';
@@ -610,10 +683,12 @@ sub _fix_windows_nt {
     my $os      = $self->[UA_OS] || q{};
     return if ( ! $os              && ! $skip_os )
         ||    (   $os ne 'windows' && ! $skip_os )
+        ||    ref $self->[UA_EXTRAS] ne 'ARRAY'
         ||      ! $self->[UA_EXTRAS][0]
-        ||        $self->[UA_EXTRAS][0] !~ m{ NT\s?(\d.*?) \z }xmsi;
+        ||        $self->[UA_EXTRAS][0] !~ m{ NT\s?(\d.*?) \z }xmsi
+    ;
     $self->[UA_EXTRAS][0] = $self->[UA_OS]; # restore
-    $self->[UA_OS] = "Windows NT $1"; # fix
+    $self->[UA_OS]        = "Windows NT $1"; # fix
     return;
 }
 
@@ -632,7 +707,7 @@ sub _parse_netscape {
     $self->[UA_VERSION_RAW] = $version;
     $self->[UA_OS]          = $buf[0] eq 'X11' ? pop @buf : shift @buf;
     $self->[UA_NAME]        = 'Netscape';
-    $self->[UA_EXTRAS]      = [ @buf ];
+    $self->[UA_EXTRAS]      = @buf ? [ @buf ] : undef;
     if ( $junk ) {
         $junk =~ s{ \[ (.+?) \] .* \z}{$1}xms;
         $self->[UA_LANG] = $junk if $junk;
@@ -673,7 +748,7 @@ sub _generic_moz_thing {
         \$self->[UA_OS], \$self->[UA_NAME], \$self->[UA_VERSION_RAW], \@extras
     );
 
-    $self->[UA_EXTRAS]      = [ @extras ] if @extras;
+    $self->[UA_EXTRAS]      = @extras ? [ @extras ] : undef;
     $self->[UA_GENERIC]     = 1;
     $self->[UA_PARSER]      = 'generic_moz_thing';
 
@@ -754,7 +829,7 @@ sub _generic_compatible {
     $self->[UA_VERSION_RAW] = $version || 0;
     $self->[UA_OS]          = $os;
     $self->[UA_LANG]        = $lang;
-    $self->[UA_EXTRAS]      = [ @extras ] if @extras;
+    $self->[UA_EXTRAS]      = @extras ? [ @extras ] : undef;
     $self->[UA_GENERIC]     = 1;
     $self->[UA_PARSER]      = 'generic_compatible';
 
@@ -773,7 +848,8 @@ sub _parse_emacs {
     my @rest = (  @{ $thing }, @moz );
     push @rest, @{ $extra } if $extra && ref $extra eq 'ARRAY';
     push @rest, ( map { split RE_SC_WS, $_ } @others ) if @others;
-    $self->[UA_EXTRAS]      = [ grep { $_ } map { $self->trim( $_ ) } @rest ];
+    my @extras = grep { $_ } map { $self->trim( $_ ) } @rest;
+    $self->[UA_EXTRAS]      = @extras ? [ @extras ] : undef;
     $self->[UA_PARSER]      = 'emacs';
     return 1;
 }
@@ -796,7 +872,7 @@ sub _parse_moz_only {
 
     $self->[UA_NAME]        = $name;
     $self->[UA_VERSION_RAW] = $version || 0;
-    $self->[UA_EXTRAS]      = [ @parts ];
+    $self->[UA_EXTRAS]      = @parts ? [ @parts ] : undef;
     $self->[UA_PARSER]      = 'moz_only';
     $self->[UA_ROBOT]       = 1 if ! $self->[UA_VERSION_RAW];
 
@@ -813,10 +889,12 @@ sub _parse_symbian {
 
     return if ! $device;
 
+    my @extras = map { split m{[\s]+}xms, $_ } @rest;
+
     @{ $self }[ UA_NAME, UA_VERSION_RAW ] = split RE_SLASH, $series, 2;
     $self->[UA_OS]     = $os;
     $self->[UA_DEVICE] = $device;
-    $self->[UA_EXTRAS] = [ map { split m{[\s]+}xms, $_ } @rest ];
+    $self->[UA_EXTRAS] = @extras ? [ @extras ] : undef;
     $self->[UA_MOBILE] = 1;
     $self->[UA_PARSER] = 'symbian';
 
@@ -835,7 +913,7 @@ sub _parse_hotjava {
             @parts = map { $self->trim( $_ ) } @parts;
             $self->[UA_OS]     = pop @parts;
             $self->[UA_LANG]   = pop @parts;
-            $self->[UA_EXTRAS] = [ @parts ];
+            $self->[UA_EXTRAS] = @parts ? [ @parts ] : undef;
         }
     }
     return 1;
@@ -847,7 +925,7 @@ sub _parse_docomo {
         my($name, $version)     = split RE_SLASH, shift @{ $thing };
         $self->[UA_NAME]        = $name;
         $self->[UA_VERSION_RAW] = $version;
-        $self->[UA_EXTRAS]      = [ @{ $thing } ];
+        $self->[UA_EXTRAS]      = @{ $thing } > 0 ? [ @{ $thing } ] : undef;
         $self->[UA_MOBILE]      = 1;
         $self->[UA_ROBOT]       = 1;
         $self->[UA_PARSER]      = 'docomo';
@@ -864,35 +942,37 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
-Parse::HTTP::UserAgent::Base::Parsers - Base class
+Parse::HTTP::UserAgent::Base::Parsers
+
+=head1 VERSION
+
+version 0.40_02
 
 =head1 DESCRIPTION
 
-This document describes version C<0.40_01> of C<Parse::HTTP::UserAgent::Base::Parsers>
-released on C<20 January 2014>.
-
-B<WARNING>: This version of the module is part of a
-developer (beta) release of the distribution and it is
-not suitable for production use.
 Internal module.
 
+=head1 NAME
+
+Parse::HTTP::UserAgent::Base::Parsers - Base class
+
 =head1 SEE ALSO
 
 L<Parse::HTTP::UserAgent>.
 
 =head1 AUTHOR
 
-Burak Gursoy <burak@cpan.org>.
+Burak Gursoy <burak@cpan.org>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009 - 2014 Burak Gursoy. All rights reserved.
+This software is copyright (c) 2009 by Burak Gursoy.
 
-=head1 LICENSE
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.16.2 or,
-at your option, any later version of Perl 5 you may have available.
 =cut
