@@ -1,10 +1,10 @@
 package Parse::HTTP::UserAgent::Constants;
-use strict;
+$Parse::HTTP::UserAgent::Constants::VERSION = '0.40_02'; # TRIAL
+
+$Parse::HTTP::UserAgent::Constants::VERSION = '0.4002';use strict;
 use warnings;
 use base qw( Exporter );
 
-our $VERSION = '0.40_01';
-
 our(@EXPORT, @EXPORT_OK, %EXPORT_TAGS);
 
 use constant LIST_ROBOTS => qw(
@@ -187,35 +187,37 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
-Parse::HTTP::UserAgent::Constants - Various constants
+Parse::HTTP::UserAgent::Constants
+
+=head1 VERSION
+
+version 0.40_02
 
 =head1 DESCRIPTION
 
-This document describes version C<0.40_01> of C<Parse::HTTP::UserAgent::Constants>
-released on C<20 January 2014>.
-
-B<WARNING>: This version of the module is part of a
-developer (beta) release of the distribution and it is
-not suitable for production use.
 Internal module
 
+=head1 NAME
+
+Parse::HTTP::UserAgent::Constants - Various constants
+
 =head1 SEE ALSO
 
 L<Parse::HTTP::UserAgent>.
 
 =head1 AUTHOR
 
-Burak Gursoy <burak@cpan.org>.
+Burak Gursoy <burak@cpan.org>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009 - 2014 Burak Gursoy. All rights reserved.
+This software is copyright (c) 2009 by Burak Gursoy.
 
-=head1 LICENSE
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself, either Perl version 5.16.2 or,
-at your option, any later version of Perl 5 you may have available.
 =cut
